import java.applet.Applet;
import java.awt.BorderLayout;
import java.util.Arrays;
import java.util.List;

import javax.swing.JFrame;

import de.jreality.plugin.JRViewer;
import de.jreality.plugin.basic.View;
import de.jreality.scene.Appearance;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.shader.CommonAttributes;
import de.jreality.tools.RotateTool;
import de.jreality.ui.viewerapp.ViewerSwitch;
import de.jreality.util.CameraUtility;
import de.raida.cad.dex.cadimport.adapter.EntityContainer;
import de.raida.cad.dex.cadimport.adapter.JRealityAdapter;
import de.raida.cad.dex.cadimport.dxf.loader.DXFImporterImplementation;
import de.raida.cad.dex.interfaces.CADImportInterface;


/**
 * Example class for loading and viewing a CAD file with Java3D.
 * @author  <a href="mailto:j.raida@gmx.net">Johannes Raida</a>
 * @version 1.0
 */
public class JRealityAdapterSample extends Applet {
	/** ID of the serializable class */
	private final static long serialVersionUID = 3L;

	/** Scene graph component where the geometry is attached */
	private SceneGraphComponent _geometrySceneGraphComponent;

	/**
	 * Constructor, creating the scene.
	 * @param fileName File name
	 */
	public JRealityAdapterSample(String fileName){
		try {
			// Load a CAD file
			//CADImportInterface cadImportInterface = new DS3ImporterImplementation();
			//CADImportInterface cadImportInterface = new BGFImporterImplementation();
			CADImportInterface cadImportInterface = new DXFImporterImplementation();
			//CADImportInterface cadImportInterface = new OBJImporterImplementation();
			//CADImportInterface cadImportInterface = new STLImporterImplementation();
			//CADImportInterface cadImportInterface = new JTImporterImplementation();
			cadImportInterface.importFile(fileName, null);

			// Convert the loaded DXF model to jReality
			JRealityAdapter jRealityAdapter = new JRealityAdapter(cadImportInterface, true, false);
			_geometrySceneGraphComponent = jRealityAdapter.getRootSceneGraphComponent();
			Appearance appearance = new Appearance();
			appearance.setAttribute(CommonAttributes.POLYGON_SHADER + "." + CommonAttributes.SMOOTH_SHADING, true);
			_geometrySceneGraphComponent.setAppearance(appearance);

			JRViewer jrViewer = new JRViewer();
			jrViewer.setContent(_geometrySceneGraphComponent);
			jrViewer.startupLocal();
			ViewerSwitch viewerSwitch = jrViewer.getPlugin(View.class).getViewer();

			// Configure the shown entities
			//setShowShading(true);
			CameraUtility.encompass(viewerSwitch);

			// Prepare the canvas
			setLayout(new BorderLayout());
			add(BorderLayout.CENTER, viewerSwitch.getViewingComponent());

			// Add some tools
			SceneGraphComponent sceneRoot = jrViewer.getViewer().getSceneRoot();
			for(int i = 0; i < sceneRoot.getChildComponentCount(); i++){
				SceneGraphComponent sceneRootChild = sceneRoot.getChildComponent(i);
				if(sceneRootChild.getName().equals("content")){
					sceneRootChild.addTool(new RotateTool());
					break;
				}
			}

			// Print some meta data
			System.out.println("\nNumber of layers: " + cadImportInterface.getLayerMetaData().size());

			System.out.println("\nExtreme values:");
			double[][] extremeValues = cadImportInterface.getExtremeValues();
			System.out.println("   Minimum: " + extremeValues[0][0] + ", " + extremeValues[0][1] + ", " + extremeValues[0][2]);
			System.out.println("   Maximum: " + extremeValues[1][0] + ", " + extremeValues[1][1] + ", " + extremeValues[1][2]);

			System.out.println("\nViewport:");
			double[][] viewPort = cadImportInterface.getViewPort();
			if(viewPort != null){
				System.out.println("   Minimum: " + viewPort[0][0] + ", " + viewPort[0][1] + ", " + viewPort[0][2]);
				System.out.println("   Maximum: " + viewPort[1][0] + ", " + viewPort[1][1] + ", " + viewPort[1][2]);
			}

			System.out.println("\nAdditional information:");
			List<String[]> modelInformation = cadImportInterface.getAdditionalModelInformation();
			for(int i = 0; i < modelInformation.size(); i++){
				String[] information = modelInformation.get(i);
				if(!information[0].equals("") && !information[1].equals("")){
					System.out.println("   " + information[0] + ": " + information[1]);
				} else if(!information[0].equals("")){
					System.out.println("   " + information[0]);
				} else if(!information[1].equals("")){
					System.out.println("   " + information[1]);
				} else {
					System.out.println();
				}
			}

			System.out.println("\nLoad information:");
			List<String[]> loadInformation = cadImportInterface.getImportInformation();
			for(int i = 0; i < loadInformation.size(); i++){
				String[] information = loadInformation.get(i);
				System.out.println("   " + information[0] + ": " + information[1]);
			}

			System.out.println("\nUnsupported entities:");
			List<String> unsupportedEntities = cadImportInterface.getUnsupportedEntities();
			for(int i = 0; i < unsupportedEntities.size(); i++){
				String unsupportedEntity = unsupportedEntities.get(i);
				System.out.println("   " + unsupportedEntity);
			}

			// Print the information about the CAD entities
			System.out.println("\nCAD entities:");
			List<EntityContainer> entityContainers = cadImportInterface.getEntities();
			for(int i = 0; ((i < entityContainers.size()) && (i != 1)); i++){
				EntityContainer entityContainer = entityContainers.get(i);

				System.out.println("  Node name:      " + entityContainer.getNodeName());
				switch(entityContainer.getType()){
					case EntityContainer.POINTS:
						System.out.println("  Entity type:    POINTS");
						break;
					case EntityContainer.POLYLINE:
						System.out.println("  Entity type:    POLYLINE");
						break;
					case EntityContainer.TRIANGLES:
						System.out.println("  Entity type:    TRIANGLES");
						break;
					default:
						System.out.println("  Entity type:    Unknown");
				}
				System.out.println("  Vertex colors:  " + Arrays.toString(entityContainer.getVertexColors()));
				System.out.println("  Vertex indices: " + Arrays.toString(entityContainer.getVertexIndices()));
				System.out.println("  Vertex normals: " + Arrays.toString(entityContainer.getVertexNormals()));
				System.out.println("  Vertices:       " + Arrays.toString(entityContainer.getVertices()));
			}

		} catch(Exception exception){
			exception.printStackTrace();
		}
	}

	/**
	 * Entry point of the class
	 * @param arguments Arguments, given by the command line
	 */
	public static void main(String[] arguments){
		if(arguments.length != 1){
			System.out.println("Wrong number of parameter!");
			System.out.println("Usage: java JRealityAdapterSample <CAD file>");
			return;
		}

		JFrame jFrame = new JFrame("Raida CAD importer example");
		jFrame.setContentPane(new JRealityAdapterSample(arguments[0]));
		jFrame.setSize(400, 400);
		jFrame.setVisible(true);
	}
}