import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Switch;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseTranslate;
import com.sun.j3d.utils.universe.SimpleUniverse;

import de.raida.cad.dex.cadimport.adapter.EntityContainer;
import de.raida.cad.dex.cadimport.adapter.Java3DAdapter;
import de.raida.cad.dex.cadimport.dxf.loader.DXFImporterImplementation;
import de.raida.cad.dex.interfaces.CADImportInterface;


/**
 * Example class for loading and viewing a CAD file with Java3D.
 * @author  <a href="mailto:j.raida@gmx.net">Johannes Raida</a>
 * @version 1.0
 */
public class Java3DAdapterSample extends Applet implements MouseMotionListener {
	/** ID of the serializable class */
	private final static long serialVersionUID = 1L;

	/** Flag, telling whether the model shall be shaded or not */
	private final boolean bShadedModel = true;

	/** Stores the last mouse position */
	private Point _lastMousePoint;

	/** Transformgroup of rotation, translation and scale */
	private TransformGroup _transformGroup;

	/**
	 * Constructor, creating the scene.
	 * @param fileName File name
	 */
	public Java3DAdapterSample(String fileName){
		try {
			// Initialize the mouse position
			_lastMousePoint = new Point(0, 0);

			// Prepare the canvas
			setLayout(new BorderLayout());
			Canvas3D canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());
			canvas3D.addMouseMotionListener(this);
			add(BorderLayout.CENTER, canvas3D);

			// Load a DXF file
			//CADImportInterface cadImportInterface = new DS3ImporterImplementation();
			//CADImportInterface cadImportInterface = new BGFImporterImplementation();
			CADImportInterface cadImportInterface = new DXFImporterImplementation();
			//CADImportInterface cadImportInterface = new OBJImporterImplementation();
			//CADImportInterface cadImportInterface = new STLImporterImplementation();
			//CADImportInterface cadImportInterface = new JTImporterImplementation();
			cadImportInterface.importFile(fileName, null);

			// Convert the loaded DXF model to Java3D
			Java3DAdapter j3dAdapter = new Java3DAdapter(cadImportInterface);

			// Activate all layers of the DXF model
			final Switch switchJ3D = j3dAdapter.getSwitch(bShadedModel, false);
			switchJ3D.setWhichChild(Switch.CHILD_MASK);
			switchJ3D.setChildMask(j3dAdapter.getLayerMask());

			// Create the BranchGroup for the geometry
			BranchGroup branchGroupGeometry = new BranchGroup();
			branchGroupGeometry.addChild(switchJ3D);

			// Create the TransformGroup for the mouse rotation / translation of the geometry
			_transformGroup = new TransformGroup();
			_transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
			_transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
			_transformGroup.addChild(branchGroupGeometry);

			MouseRotate mouseRotate = new MouseRotate();
			mouseRotate.setTransformGroup(_transformGroup);
			mouseRotate.setSchedulingBounds(new BoundingBox());

			MouseTranslate mouseTranslate = new MouseTranslate();
			mouseTranslate.setTransformGroup(_transformGroup);
			mouseTranslate.setSchedulingBounds(new BoundingBox());

			branchGroupGeometry.addChild(mouseRotate);
			branchGroupGeometry.addChild(mouseTranslate);

			branchGroupGeometry.compile();

			// Create the root node, which is put into the SimpleUniverse
			BranchGroup branchGroupBase = new BranchGroup();
			branchGroupBase.addChild(_transformGroup);

			// Possibly add light to the root node
			if(bShadedModel){
				AmbientLight ambientLight = new AmbientLight(new Color3f(0.2f, 0.2f, 0.2f));
				ambientLight.setInfluencingBounds(new BoundingBox());
				branchGroupBase.addChild(ambientLight);

				DirectionalLight directionalLight = new DirectionalLight(new Color3f(1.0f, 1.0f, 1.0f), new Vector3f(0.0f, 0.0f, -1.0f));
				directionalLight.setInfluencingBounds(new BoundingBox());
				branchGroupBase.addChild(directionalLight);
			}

			// Create the centering transformation
			TransformGroup centeringTransformGroup = new TransformGroup();
			centeringTransformGroup.addChild(branchGroupBase);

			BranchGroup topBranchGroup = new BranchGroup();
			topBranchGroup.addChild(centeringTransformGroup);

			// Create the SimpleUniverse
			SimpleUniverse simpleUniverse = new SimpleUniverse(canvas3D);
			simpleUniverse.getViewingPlatform().setNominalViewingTransform();
			simpleUniverse.addBranchGraph(topBranchGroup);

			// Print some meta data
			System.out.println("\nNumber of layers: " + cadImportInterface.getLayerMetaData().size());

			System.out.println("\nExtreme values:");
			double[][] extremeValues = cadImportInterface.getExtremeValues();
			System.out.println("   Minimum: " + extremeValues[0][0] + ", " + extremeValues[0][1] + ", " + extremeValues[0][2]);
			System.out.println("   Maximum: " + extremeValues[1][0] + ", " + extremeValues[1][1] + ", " + extremeValues[1][2]);

			System.out.println("\nViewport:");
			double[][] viewPort = cadImportInterface.getViewPort();
			if(viewPort != null){
				System.out.println("   Minimum: " + viewPort[0][0] + ", " + viewPort[0][1] + ", " + viewPort[0][2]);
				System.out.println("   Maximum: " + viewPort[1][0] + ", " + viewPort[1][1] + ", " + viewPort[1][2]);
			}

			System.out.println("\nAdditional information:");
			ArrayList<String[]> modelInformation = cadImportInterface.getAdditionalModelInformation();
			for(int i = 0; i < modelInformation.size(); i++){
				String[] information = modelInformation.get(i);
				if(!information[0].equals("") && !information[1].equals("")){
					System.out.println("   " + information[0] + ": " + information[1]);
				} else if(!information[0].equals("")){
					System.out.println("   " + information[0]);
				} else if(!information[1].equals("")){
					System.out.println("   " + information[1]);
				} else {
					System.out.println();
				}
			}

			System.out.println("\nLoad information:");
			ArrayList<String[]> loadInformation = cadImportInterface.getImportInformation();
			for(int i = 0; i < loadInformation.size(); i++){
				String[] information = loadInformation.get(i);
				System.out.println("   " + information[0] + ": " + information[1]);
			}

			System.out.println("\nUnsupported entities:");
			ArrayList<String> unsupportedEntities = cadImportInterface.getUnsupportedEntities();
			for(int i = 0; i < unsupportedEntities.size(); i++){
				String unsupportedEntity = unsupportedEntities.get(i);
				System.out.println("   " + unsupportedEntity);
			}

			// Print the information about the CAD entities
			System.out.println("\nCAD entities:");
			ArrayList<EntityContainer> entityContainers = cadImportInterface.getEntities();
			for(int i = 0; ((i < entityContainers.size()) && (i != 1)); i++){
				EntityContainer entityContainer = entityContainers.get(i);

				System.out.println("  Modelspace: " + entityContainer.isInModelSpace());

				if(entityContainer.getType() == EntityContainer.LIST){
					double[] vertices = entityContainer.getVertices();
					double[] normals = entityContainer.getNormals();
					int[] indices = entityContainer.getIndices();
					double[] colors = entityContainer.getColors();

					System.out.println("  Vertices: " + vertices.length);
					System.out.println("  Normals:  " + normals.length);
					System.out.println("  Indices:  " + indices.length);
					System.out.println("  Colors:   " + colors.length);

					int numberOfFaces = indices.length / 3;
					System.out.println("  Faces:    " + numberOfFaces);

					for(int j = 0; j < numberOfFaces; j++){
						System.out.println("\n  Face " +  (j + 1) + ":");
						int index1 = indices[(j * 3) + 0];
						int index2 = indices[(j * 3) + 1];
						int index3 = indices[(j * 3) + 2];

						double[] vertex1 = new double[]{vertices[(index1 * 3) + 0], vertices[(index1 * 3) + 1], vertices[(index1 * 3) + 2]};
						double[] vertex2 = new double[]{vertices[(index2 * 3) + 0], vertices[(index2 * 3) + 1], vertices[(index2 * 3) + 2]};
						double[] vertex3 = new double[]{vertices[(index3 * 3) + 0], vertices[(index3 * 3) + 1], vertices[(index3 * 3) + 2]};
						System.out.println("   ... vertex 1: " +  vertex1[0] + ", " + vertex1[1] + ", " + vertex1[2]);
						System.out.println("   ... vertex 2: " +  vertex2[0] + ", " + vertex2[1] + ", " + vertex2[2]);
						System.out.println("   ... vertex 3: " +  vertex3[0] + ", " + vertex3[1] + ", " + vertex3[2]);

						double[] normal1 = new double[]{normals[(index1 * 3) + 0], normals[(index1 * 3) + 1], normals[(index1 * 3) + 2]};
						double[] normal2 = new double[]{normals[(index2 * 3) + 0], normals[(index2 * 3) + 1], normals[(index2 * 3) + 2]};
						double[] normal3 = new double[]{normals[(index3 * 3) + 0], normals[(index3 * 3) + 1], normals[(index3 * 3) + 2]};
						System.out.println("   ... normal 1: " +  normal1[0] + ", " + normal1[1] + ", " + normal1[2]);
						System.out.println("   ... normal 2: " +  normal1[0] + ", " + normal2[1] + ", " + normal2[2]);
						System.out.println("   ... normal 3: " +  normal2[0] + ", " + normal3[1] + ", " + normal3[2]);

						double[] color = new double[]{colors[(j * 3) + 0], colors[(j * 3) + 1], colors[(j * 3) + 2]};
						System.out.println("   ... color: " +  color[0] + ", " + color[1] + ", " + color[2]);

						if(j == 1) break;
					}
					System.out.println("\n  ... skipping other entities ...");

				} else {
					Color[] colors = entityContainer.getColor();
					for(Color color : colors){
						System.out.println("  Color: " + color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
					}

					ArrayList<double[][]> coordinates = entityContainer.getShadingCoordinates();
					for(int j = 0; j < coordinates.size(); j++){
						double[][] pointsOfEntity = coordinates.get(j);
						for(int k = 0; k < pointsOfEntity.length; k++){
							switch(pointsOfEntity.length){
								// Handle points
								case 1:
									System.out.println("  Point: " + pointsOfEntity[0][0] + ", " + pointsOfEntity[0][1] + ", " + pointsOfEntity[0][2]);
									break;

								// Handle lines
								case 2:
									System.out.println("  Line: " + pointsOfEntity[0][0] + ", " + pointsOfEntity[0][1] + ", " + pointsOfEntity[0][2]);
									System.out.println("        " + pointsOfEntity[1][0] + ", " + pointsOfEntity[1][1] + ", " + pointsOfEntity[1][2]);
									break;

								// Handle triangles
								case 3:
									System.out.println("  Triangle: " + pointsOfEntity[0][0] + ", " + pointsOfEntity[0][1] + ", " + pointsOfEntity[0][2]);
									System.out.println("            " + pointsOfEntity[1][0] + ", " + pointsOfEntity[1][1] + ", " + pointsOfEntity[1][2]);
									System.out.println("            " + pointsOfEntity[2][0] + ", " + pointsOfEntity[2][1] + ", " + pointsOfEntity[2][2]);
									break;

								// Handle faces
								case 4:
									System.out.println("  Face: " + pointsOfEntity[0][0] + ", " + pointsOfEntity[0][1] + ", " + pointsOfEntity[0][2]);
									System.out.println("        " + pointsOfEntity[1][0] + ", " + pointsOfEntity[1][1] + ", " + pointsOfEntity[1][2]);
									System.out.println("        " + pointsOfEntity[2][0] + ", " + pointsOfEntity[2][1] + ", " + pointsOfEntity[2][2]);
									System.out.println("        " + pointsOfEntity[3][0] + ", " + pointsOfEntity[3][1] + ", " + pointsOfEntity[3][2]);
									break;

								// Handle the rest
								default:
									System.err.println("ERROR: Internal warning: Unhandled number of coordinates found: " + pointsOfEntity.length);
							}
						}
						System.out.println(" ... skipping other entities ...");
						break;
					}
				}
			}

		} catch(Exception exception){
			exception.printStackTrace();
		}
	}

	/**
	 * Called if the mouse is moved while a mouse key is pressed.
	 * @param mouseEvent Thrown mouse event
	 */
	public void mouseDragged(MouseEvent mouseEvent){
		if((mouseEvent.getModifiers() & InputEvent.BUTTON2_MASK) != 0){
			int mouseX = mouseEvent.getX();
			int mouseY = mouseEvent.getY();

			Transform3D zoomTrafo = new Transform3D();
			Transform3D modelTrafo = new Transform3D();

			_transformGroup.getTransform(modelTrafo);

			if(_lastMousePoint.y > mouseY){
				zoomTrafo.set(1.05f);

			} else if(_lastMousePoint.y < mouseY){
				zoomTrafo.set(0.95f);
			}

			modelTrafo.mul(zoomTrafo);
			_transformGroup.setTransform(modelTrafo);

			_lastMousePoint.setLocation(mouseX, mouseY);
		}
	}

	/**
	 * Called if the mouse is moved while no mouse key is pressed.
	 * @param mouseEvent Thrown mouse event
	 */
	public void mouseMoved(MouseEvent mouseEvent){}

	/**
	 * Entry point of the class
	 * @param arguments Arguments, given by the command line
	 */
	public static void main(String[] arguments){
		if(arguments.length != 1){
			System.out.println("Wrong number of parameter!");
			System.out.println("Usage: java Java3DAdapterSample <CAD file>");
			return;
		}

		new MainFrame(new Java3DAdapterSample(arguments[0]), 400, 400);
	}
}